@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.h 487 2005-08-04 18:28:09Z svn $
+ * $Id: UploaderConfig.h 554 2005-09-01 11:49:36Z svn $
  *****************************************************************************/
 
 #ifndef UPLOADERCONFIG_H
@@ -40,6 +40,7 @@
 
 #include "apr_pools.h"
 #include "apr_time.h"
+#include "apr_global_mutex.h"
 #ifdef LOCK_TYPE_MUTEX
 #include "apr_thread_mutex.h"
 #else
@@ -87,6 +88,7 @@ public:
     apr_size_t max_file_size;
     apr_size_t per_page_item_number;
     apr_size_t remove_item_number;
+    apr_global_mutex_t *glock;
 
     UploadItemList *item_list;
     TemplateExecutor *texecutor;
@@ -95,18 +97,20 @@ public:
     UploaderConfig();
     ~UploaderConfig();
 
-    void init(apr_pool_t *pool, apr_pool_t *tmp_pool);
+    bool init(apr_pool_t *pool, apr_pool_t *tmp_pool);
     void finalize();
 
     page_template *get_view_template(apr_pool_t *pool);
     page_template *get_download_template(apr_pool_t *pool);
     page_template *get_thumb_template(apr_pool_t *pool);
     page_template *get_error_template(apr_pool_t *pool);
+
     UploadItemReader *get_ureader(apr_pool_t *pool);
     UploadItemWriter *get_uwriter(apr_pool_t *pool);
     apr_status_t read_lock();
     apr_status_t write_lock();
-    apr_status_t unlock();
+    apr_status_t read_unlock();
+    apr_status_t write_unlock();
 
 private:
     static const apr_uint64_t TOTAL_FILE_SIZE_LIMIT;
@@ -135,6 +139,7 @@ private:
 #endif
     volatile bool is_init;
 
+    void exec_init(apr_pool_t *pool, apr_pool_t *tmpl_pool);
     void validate();
 #if APR_HAS_THREADS
     void init_lock(apr_pool_t *pool, apr_pool_t *tmp_pool);
@@ -144,7 +149,7 @@ private:
     void load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
                        const char *template_path, page_template *tmpl);
 
-    static apr_time_t get_mtime(apr_pool_t *pool, const char *file_path);
+    static apr_time_t get_mtime(apr_pool_t *pool, const char *path);
 };
 
 #endif
