@@ -24,32 +24,22 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.cpp 487 2005-08-04 18:28:09Z svn $
+ * $Id: UploaderConfig.cpp 554 2005-09-01 11:49:36Z svn $
  *****************************************************************************/
 
 #include "UploaderConfig.h"
+#include "Auxiliary.h"
 #include "Misc.h"
 
 #include "apr_strings.h"
 
-#ifdef WIN32
-#include <Windows.h>
-#else
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
-#endif
-
 #ifdef DEBUG
 #include <iostream>
 #endif
 
 #undef DELETE
 #define DELETE(ptr) if (ptr != NULL) { delete ptr; ptr = NULL;}
-#define CLEAR(pool) if (pool != NULL) { apr_pool_clear(pool); pool = NULL;}
+#define DESTROY(pool) if (pool != NULL) { apr_pool_destroy(pool); pool = NULL;}
 
 const apr_uint64_t UploaderConfig::TOTAL_FILE_SIZE_LIMIT= 10*1024*1024;
 const apr_size_t UploaderConfig::TOTAL_FILE_NUMBER_LIMIT= 1000;
@@ -57,7 +47,7 @@ const apr_size_t UploaderConfig::MAX_FORM_FILE_SIZE     = 1*1024*1024;
 const apr_size_t UploaderConfig::MAX_FORM_TEXT_SIZE     = 1024;
 const apr_size_t UploaderConfig::MAX_FORM_ITEM_NUMBER   = 5;
 const apr_size_t UploaderConfig::PER_PAGE_ITEM_NUMBER   = 50;
-const apr_size_t UploaderConfig::ITEM_RELOAD_THRESHOLD  = 100;
+const apr_size_t UploaderConfig::ITEM_RELOAD_THRESHOLD  = 10;
 
 static apr_status_t finalize_config(void *data);
 #if APR_HAS_THREADS
@@ -78,11 +68,12 @@ UploaderConfig::UploaderConfig()
       download_tmpl_path(NULL),
       thumb_tmpl_path(NULL),
       error_tmpl_path(NULL),
-      total_file_size_limit( TOTAL_FILE_SIZE_LIMIT),
-      total_file_number_limit( TOTAL_FILE_NUMBER_LIMIT),
+      total_file_size_limit(TOTAL_FILE_SIZE_LIMIT),
+      total_file_number_limit(TOTAL_FILE_NUMBER_LIMIT),
       max_file_size(MAX_FORM_FILE_SIZE),
-      per_page_item_number( PER_PAGE_ITEM_NUMBER),
+      per_page_item_number(PER_PAGE_ITEM_NUMBER),
       remove_item_number(0),
+      glock(NULL),
       item_list(NULL),
       texecutor(NULL),
       uparser(NULL),
@@ -108,34 +99,17 @@ UploaderConfig::~UploaderConfig()
     finalize();
 }
 
-void UploaderConfig::init(apr_pool_t *pool, apr_pool_t *tmp_pool)
+bool UploaderConfig::init(apr_pool_t *pool, apr_pool_t *tmpl_pool)
 {
     if (LIKELY(is_init)) {
-        return update_list(pool, tmp_pool);
+        update_list(pool, tmpl_pool);
+
+        return false;
+    } else {
+        exec_init(pool, tmpl_pool);
+
+        return true;
     }
-
-    validate();
-
-#if APR_HAS_THREADS
-    init_lock(pool, tmp_pool);
-#endif
-    load_list(pool);
-
-    view_tmpl = new page_template();
-    download_tmpl = new page_template();
-#ifdef MAKE_THUMBNAIL
-    thumb_tmpl = new page_template();
-#endif
-    error_tmpl = new page_template();
-
-    texecutor = new TemplateExecutor();
-    uparser = new UploadParser(tmp_dir, MAX_FORM_TEXT_SIZE,
-                               max_file_size, MAX_FORM_ITEM_NUMBER,
-                               UploadItem::ITEM_HEADER_SIZE);
-
-    apr_pool_cleanup_register(tmp_pool, this, apr_pool_cleanup_null, finalize_config);
-
-    is_init = true;
 }
 
 void UploaderConfig::finalize()
@@ -151,13 +125,13 @@ void UploaderConfig::finalize()
 #endif
     DELETE(error_tmpl);
 
-    CLEAR(list_pool);
-    CLEAR(view_tmpl_pool);
-    CLEAR(download_tmpl_pool);
+    DESTROY(list_pool);
+    DESTROY(view_tmpl_pool);
+    DESTROY(download_tmpl_pool);
 #ifdef MAKE_THUMBNAIL
-    CLEAR(thumb_tmpl_pool);
+    DESTROY(thumb_tmpl_pool);
 #endif
-    CLEAR(error_tmpl_pool);
+    DESTROY(error_tmpl_pool);
 }
 
 UploaderConfig::page_template *UploaderConfig::get_view_template(apr_pool_t *pool)
@@ -210,45 +184,80 @@ apr_status_t UploaderConfig::read_lock()
 {
 #if APR_HAS_THREADS
 #ifdef LOCK_TYPE_MUTEX
-    return apr_thread_mutex_lock(lock);
+    apr_thread_mutex_lock(lock);
 #else
-    return apr_thread_rwlock_rdlock(lock);
+    apr_thread_rwlock_rdlock(lock);
+#endif
 #endif
-#else
     return APR_SUCCESS;
-#endif
 }
 
 apr_status_t UploaderConfig::write_lock()
 {
 #if APR_HAS_THREADS
 #ifdef LOCK_TYPE_MUTEX
-    return apr_thread_mutex_lock(lock);
+    apr_thread_mutex_lock(lock);
 #else
-    return apr_thread_rwlock_wrlock(lock);
+    apr_thread_rwlock_wrlock(lock);
 #endif
-#else
-    return APR_SUCCESS;
 #endif
+    return apr_global_mutex_lock(glock);
 }
 
-apr_status_t UploaderConfig::unlock()
+apr_status_t UploaderConfig::read_unlock()
 {
 #if APR_HAS_THREADS
 #ifdef LOCK_TYPE_MUTEX
-    return apr_thread_mutex_unlock(lock);
+    apr_thread_mutex_unlock(lock);
 #else
-    return apr_thread_rwlock_unlock(lock);
+    apr_thread_rwlock_unlock(lock);
+#endif
 #endif
-#else
     return APR_SUCCESS;
+}
+
+apr_status_t UploaderConfig::write_unlock()
+{
+#if APR_HAS_THREADS
+#ifdef LOCK_TYPE_MUTEX
+    apr_thread_mutex_unlock(lock);
+#else
+    apr_thread_rwlock_unlock(lock);
 #endif
+#endif
+    return apr_global_mutex_unlock(glock);
 }
 
 
 /******************************************************************************
  * private ᥽å
  *****************************************************************************/
+void UploaderConfig::exec_init(apr_pool_t *pool, apr_pool_t *tmpl_pool)
+{
+    validate();
+
+#if APR_HAS_THREADS
+    init_lock(pool, tmpl_pool);
+#endif
+    load_list(pool);
+
+    view_tmpl = new page_template();
+    download_tmpl = new page_template();
+#ifdef MAKE_THUMBNAIL
+    thumb_tmpl = new page_template();
+#endif
+    error_tmpl = new page_template();
+
+    texecutor = new TemplateExecutor();
+    uparser = new UploadParser(tmp_dir, MAX_FORM_TEXT_SIZE,
+                               max_file_size, MAX_FORM_ITEM_NUMBER,
+                               sizeof(UploadItem::header));
+
+    apr_pool_cleanup_register(tmpl_pool, this, apr_pool_cleanup_null, finalize_config);
+
+    is_init = true;
+}
+
 void UploaderConfig::validate()
 {
     if (file_dir            == NULL) throw "FileDirectory is not specified.";
@@ -264,15 +273,15 @@ void UploaderConfig::validate()
 }
 
 #if APR_HAS_THREADS
-void UploaderConfig::init_lock(apr_pool_t *pool, apr_pool_t *tmp_pool)
+void UploaderConfig::init_lock(apr_pool_t *pool, apr_pool_t *tmpl_pool)
 {
 #ifdef LOCK_TYPE_MUTEX
     if (apr_thread_mutex_create(&lock, APR_THREAD_MUTEX_DEFAULT, pool) != APR_SUCCESS) {
-        throw "Can not create lock.";
+        throw "Can not create thread mutex.";
     }
 #else
     if (apr_thread_rwlock_create(&lock, pool) != APR_SUCCESS) {
-        throw "Can not create lock.";
+        throw "Can not create rwlock.";
     }
 #endif
 
@@ -283,7 +292,7 @@ void UploaderConfig::init_lock(apr_pool_t *pool, apr_pool_t *tmp_pool)
 void UploaderConfig::load_list(apr_pool_t *pool)
 {
     DELETE(item_list);
-    CLEAR(list_pool);
+    DESTROY(list_pool);
 
     if (apr_pool_create(&list_pool, pool) != APR_SUCCESS) {
         throw "åץɥƥΥꥹѤΥݤ˼Ԥޤ";
@@ -293,39 +302,17 @@ void UploaderConfig::load_list(apr_pool_t *pool)
                                      total_file_number_limit, total_file_size_limit);
 }
 
-void UploaderConfig::update_list(apr_pool_t *pool, apr_pool_t *tmp_pool)
+void UploaderConfig::update_list(apr_pool_t *pool, apr_pool_t *tmpl_pool)
 {
-    if (get_mtime(tmp_pool, file_dir) > item_list->get_mtime()) {
+    if (UNLIKELY(item_list->is_need_update())) {
         try {
             write_lock();
 
             load_list(pool);
 
-            apr_time_t mtime = item_list->get_mtime();
-
-#ifdef WIN32
-            FILETIME time;
-            HANDLE file = CreateFile(file_dir,
-                                     GENERIC_WRITE, 0, 0, OPEN_EXISTING,
-                                     FILE_ATTRIBUTE_NORMAL, NULL);
-
-            mtime += APR_TIME_C(11644473600000000);
-            mtime *= 10;
-            time.dwLowDateTime = static_cast<DWORD>(mtime & 0xFFFFFFFF);
-            time.dwHighDateTime = static_cast<DWORD>(mtime >> 32);
-
-            if (file != INVALID_HANDLE_VALUE) {
-                SetFileTime(file, NULL, &time, &time);
-                CloseHandle(file);
-            }
-#elif HAVE_UTIME
-            struct utimbuf time;
-            time.modtime = time.actime = mtime/1000000;
-            utime(file_dir, &time);
-#endif
-            unlock();
+            write_unlock();
         } catch(const char *) {
-            unlock();
+            write_unlock();
             throw;
         }
     } else if (remove_item_number++ == ITEM_RELOAD_THRESHOLD) {
@@ -335,9 +322,9 @@ void UploaderConfig::update_list(apr_pool_t *pool, apr_pool_t *tmp_pool)
             load_list(pool);
             remove_item_number = 0;
 
-            unlock();
+            write_unlock();
         } catch(const char *) {
-            unlock();
+            write_unlock();
             throw;
         }
     }
@@ -367,13 +354,13 @@ void UploaderConfig::load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
         }
 
         DELETE(tmpl->imap);
-        CLEAR(*tmpl_pool);
+        DESTROY(*tmpl_pool);
 
         if (apr_pool_create(tmpl_pool, pool) != APR_SUCCESS) {
             throw "ƥץ졼ѤΥݤ˼Ԥޤ";
         }
 
-        TemplateLexer lexer(*tmpl_pool, reinterpret_cast<const char *>(file_map->mm),
+        TemplateLexer lexer(*tmpl_pool, static_cast<const char *>(file_map->mm),
                             static_cast<apr_size_t>(info.size));
         TemplateParser parser(*tmpl_pool);
 
@@ -394,17 +381,19 @@ void UploaderConfig::load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
     }
 }
 
-apr_time_t UploaderConfig::get_mtime(apr_pool_t *pool, const char *file_path)
+apr_time_t UploaderConfig::get_mtime(apr_pool_t *pool, const char *path)
 {
-    apr_finfo_t info;
+    apr_time_t time;
 
-    if (UNLIKELY(apr_stat(&info, file_path, APR_FINFO_MTIME, pool) != APR_SUCCESS)) {
+    time = mtime(pool, path);
+
+    if (UNLIKELY(time == 0)) {
         throw apr_psprintf(pool,
                            "Failed to get information of file/directory (%s).",
-                           file_path);
+                           path);
     }
 
-    return info.mtime;
+    return time;
 }
 
 
@@ -415,7 +404,7 @@ apr_status_t finalize_config(void *data)
 {
     UploaderConfig *config;
 
-    config = reinterpret_cast<UploaderConfig *>(data);
+    config = static_cast<UploaderConfig *>(data);
     config->finalize();
 
     return APR_SUCCESS;
@@ -427,12 +416,12 @@ apr_status_t finalize_lock(void *data)
 #ifdef LOCK_TYPE_MUTEX
     apr_thread_mutex_t *lock;
 
-    lock = reinterpret_cast<apr_thread_mutex_t *>(data);
+    lock = static_cast<apr_thread_mutex_t *>(data);
     apr_thread_mutex_destroy(lock);
 #else
     apr_thread_rwlock_t *lock;
 
-    lock = reinterpret_cast<apr_thread_rwlock_t *>(data);
+    lock = static_cast<apr_thread_rwlock_t *>(data);
     apr_thread_rwlock_destroy(lock);
 #endif
 
